Package: rsamplr
Title: Sampling Algorithms and Spatially Balanced Sampling
Version: 0.2.0
Authors@R: c(
    person("Wilmer", "Prentius", , "wilmer.prentius@slu.se",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3561-290X")),
    person("Anton", "Grafström", role = "ctb",
           comment = c(ORCID = "0000-0002-4345-4024")),
    person("Authors of the dependent Rust crates", role = "aut",
           comment = "see inst/AUTHORS file")
  )
Description: 
    Fast tools for unequal probability sampling in multi-dimensional spaces, implemented in Rust for high performance.
    The package offers a wide range of methods, including Sampford (Sampford, 1967, <doi:10.1093/biomet/54.3-4.499>) and correlated Poisson sampling (Bondesson and Thorburn, 2008, <doi:10.1111/j.1467-9469.2008.00596.x>), pivotal sampling (Deville and Tillé, 1998, <doi:10.1093/biomet/91.4.893>), and balanced sampling such as the cube method (Deville and Tillé, 2004, <doi:10.1093/biomet/91.4.893>) to ensure auxiliary totals are respected.
    Spatially balanced approaches, including the local pivotal method (Grafström et al., 2012, <doi:10.1111/j.1541-0420.2011.01699.x>), spatially correlated Poisson sampling (Grafström, 2012, <doi:10.1016/j.jspi.2011.07.003>), and locally correlated Poisson sampling (Prentius, 2024, <doi:10.1002/env.2832>), provide efficient designs when the target variable is linked to auxiliary information.
URL: https://www.envisim.se/, https://github.com/envisim/rust-samplr/
BugReports: https://github.com/envisim/rust-samplr/issues
License: AGPL-3
Encoding: UTF-8
Language: en-GB
Depends: R (>= 4.5)
RoxygenNote: 7.3.3
SystemRequirements: Cargo (Rust's package manager), rustc >= 1.84.1
NeedsCompilation: yes
Packaged: 2026-03-30 14:57:44 UTC; wpmg
Author: Wilmer Prentius [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3561-290X>),
  Anton Grafström [ctb] (ORCID: <https://orcid.org/0000-0002-4345-4024>),
  Authors of the dependent Rust crates [aut] (see inst/AUTHORS file)
Maintainer: Wilmer Prentius <wilmer.prentius@slu.se>
Repository: CRAN
Date/Publication: 2026-03-31 06:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 02:24:56 UTC; unix
Archs: rsamplr.so.dSYM

Package: rsamplr
Title: Sampling Algorithms and Spatially Balanced Sampling
Version: 0.2.0
Authors@R: c(
    person("Wilmer", "Prentius", , "wilmer.prentius@slu.se",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3561-290X")),
    person("Anton", "Grafström", role = "ctb",
           comment = c(ORCID = "0000-0002-4345-4024")),
    person("Authors of the dependent Rust crates", role = "aut",
           comment = "see inst/AUTHORS file")
  )
Description: 
    Fast tools for unequal probability sampling in multi-dimensional spaces, implemented in Rust for high performance.
    The package offers a wide range of methods, including Sampford (Sampford, 1967, <doi:10.1093/biomet/54.3-4.499>) and correlated Poisson sampling (Bondesson and Thorburn, 2008, <doi:10.1111/j.1467-9469.2008.00596.x>), pivotal sampling (Deville and Tillé, 1998, <doi:10.1093/biomet/91.4.893>), and balanced sampling such as the cube method (Deville and Tillé, 2004, <doi:10.1093/biomet/91.4.893>) to ensure auxiliary totals are respected.
    Spatially balanced approaches, including the local pivotal method (Grafström et al., 2012, <doi:10.1111/j.1541-0420.2011.01699.x>), spatially correlated Poisson sampling (Grafström, 2012, <doi:10.1016/j.jspi.2011.07.003>), and locally correlated Poisson sampling (Prentius, 2024, <doi:10.1002/env.2832>), provide efficient designs when the target variable is linked to auxiliary information.
URL: https://www.envisim.se/, https://github.com/envisim/rust-samplr/
BugReports: https://github.com/envisim/rust-samplr/issues
License: AGPL-3
Encoding: UTF-8
Language: en-GB
Depends: R (>= 4.5)
RoxygenNote: 7.3.3
SystemRequirements: Cargo (Rust's package manager), rustc >= 1.84.1
NeedsCompilation: yes
Packaged: 2026-03-30 14:57:44 UTC; wpmg
Author: Wilmer Prentius [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3561-290X>),
  Anton Grafström [ctb] (ORCID: <https://orcid.org/0000-0002-4345-4024>),
  Authors of the dependent Rust crates [aut] (see inst/AUTHORS file)
Maintainer: Wilmer Prentius <wilmer.prentius@slu.se>
Repository: CRAN
Date/Publication: 2026-03-31 06:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 22:02:48 UTC; unix
Archs: rsamplr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("$<-",savvy_rsamplr__sealed)
S3method("[[<-",savvy_rsamplr__sealed)
S3method(draw,dbd)
export(balance_deviation)
export(brewer)
export(conditional_poisson)
export(cps)
export(cube)
export(cube_stratified)
export(dbd_circular)
export(dbd_circular_iter)
export(dbd_tc)
export(dbd_tc_iter)
export(draw)
export(lcps)
export(local_cube)
export(local_cube_stratified)
export(local_mean_variance)
export(lpm_1)
export(lpm_1s)
export(lpm_2)
export(lpm_2_hierarchical)
export(pareto)
export(pips_from_vector)
export(poisson)
export(rpm)
export(sampford)
export(sample_to_indicator)
export(scps)
export(spatial_balance_all)
export(spatial_balance_all_equal)
export(spatial_balance_energy)
export(spatial_balance_energy_equal)
export(spatial_balance_local)
export(spatial_balance_local_equal)
export(spatial_balance_voronoi)
export(spatial_balance_voronoi_equal)
export(spm)
export(systematic)
export(systematic_random_order)
useDynLib(rsamplr, .registration = TRUE)

#' Inclusion probabilities proportional-to-size
#'
#' @description
#' Computes the first-order inclusion probabilities from a vector of positive numbers,
#' for an inclusion probabilities proportional-to-size design.
#'
#' @param values A vector of positive numbers
#' @param sample_size The wanted sample size
#'
#' @return A vector of inclusion probabilities proportional-to-size.
#'
#' @examples
#' set.seed(12345);
#' N = 1000;
#' n = 100;
#' x = matrix(runif(N * 2), ncol = 2);
#' prob = pips_from_vector(x[, 1], n);
#' s = lpm_2(prob, x);
#' plot(x[, 1], x[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' @export
pips_from_vector = function(values, sample_size) {
  values = as.double(values);
  N = length(values);
  sample_size = as.integer(sample_size);

  if (length(sample_size) != 1 || sample_size > N || sample_size < 0)
    stop("n must be integer in [0, N]");

  if (sample_size == N)
    return(rep(1.0, N));
  if (sample_size == 0)
    return(rep(0.0, N));

  rust_pips_from_values(
    values,
    sample_size
  )
}

#' Spatial balance measure
#'
#' @name Spatial balance measure
#' @rdname spatial_balance_measure
#' @description
#' Calculates the spatial balance of a sample.
#'
#' @param sample A vector of sample indices.
#' @param probabilities A vector of inclusion probabilities.
#' @param spread_mat A matrix of spreading covariates.
#' @param balance_probabilities If `true` (default), includes the vector of inclusion probabilities
#' as a balancing variable.
#' @param sample_size The sample size
#'
#' @returns the measure, or in case of `balance_deviation`, the vector of deviations.
#'
#' @references
#' Stevens Jr, D. L., & Olsen, A. R. (2004).
#' Spatially balanced sampling of natural resources.
#' Journal of the American statistical Association, 99(465), 262-278.
#'
#' Grafström, A., Lundström, N.L.P. & Schelin, L. (2012).
#' Spatially balanced sampling through the Pivotal method.
#' Biometrics 68(2), 514-520.
#'
#' Prentius, W., & Grafström, A. (2024).
#' How to find the best sampling design: A new measure of spatial balance.
#' Environmetrics, 35(7), e2878.
#'
#' @examples
#' set.seed(12345);
#' N = 500;
#' n = 70;
#' prob = rep(n / N, N);
#' xs = matrix(runif(N * 2), ncol = 2);
#'
#' s = lpm_2(prob, xs);
#' spatial_balance_voronoi(s, prob, xs);
#' spatial_balance_local(s, prob, xs, TRUE);
#' spatial_balance_local(s, prob, xs, FALSE);
#' spatial_balance_energy(s, prob, xs);
#' balance_deviation(s, prob, xs);
#'
#' \donttest{
#' # Compare SRS
#' r = 1000L;
#' sb_v = matrix(0.0, r, 2L);
#' sb_l = matrix(0.0, r, 4L);
#' sb_e = matrix(0.0, r, 2L);
#' bal = matrix(0.0, r, 2L * ncol(xs));
#'
#' for (i in seq_len(r)) {
#'   s1 = lpm_2(prob, xs);
#'   s2 = sample(N, n);
#'   sb_v[i, ] = c(
#'     spatial_balance_voronoi(s1, prob, xs),
#'     spatial_balance_voronoi(s2, prob, xs)
#'   );
#'   sb_l[i, ] = c(
#'     spatial_balance_local(s1, prob, xs, TRUE),
#'     spatial_balance_local(s2, prob, xs, TRUE),
#'     spatial_balance_local(s1, prob, xs, FALSE),
#'     spatial_balance_local(s2, prob, xs, FALSE)
#'   );
#'   sb_e[i, ] = c(
#'     spatial_balance_energy(s1, prob, xs),
#'     spatial_balance_energy(s2, prob, xs)
#'   );
#'   bal[i, ] = c(
#'     balance_deviation(s1, prob, xs),
#'     balance_deviation(s2, prob, xs)
#'   );
#' }
#'
#' # Spatial balance measure (voronoi), LPM vs SRS
#' print(colMeans(sb_v));
#' # Spatial balance measure (local), LPM vs SRS
#' print(colMeans(sb_l));
#' # Spatial balance measure (energy), LPM vs SRS
#' print(colMeans(sb_e));
#' # Abs. balance deviation, LPM vs SRS
#' print(colMeans(abs(bal)));
#' }
#'
NULL

.spatial_balance_measure_wrapper = function(method, sample, probabilities, spread_mat) {
  rust_spatial_balance_measure(
    as.integer(sample),
    as.double(probabilities),
    as.matrix(spread_mat),
    method
  )
}
.spatial_balance_measure_wrapper_equal = function(method, sample, sample_size, spread_mat) {
  rust_spatial_balance_measure_equal(
    as.integer(sample),
    as.integer(sample_size),
    as.matrix(spread_mat),
    method
  )
}

#' @describeIn spatial_balance_measure Local spatial balance
#' @export
spatial_balance_local = function(sample, probabilities, spread_mat, balance_probabilities = TRUE) {
  if (balance_probabilities == FALSE) {
    .spatial_balance_measure_wrapper("local2", sample, probabilities, spread_mat)
  } else {
    .spatial_balance_measure_wrapper("local", sample, probabilities, spread_mat)
  }
}

#' @describeIn spatial_balance_measure Local spatial balance
#' @export
spatial_balance_local_equal = function(
  sample,
  spread_mat,
  sample_size = length(sample),
  balance_probabilities = TRUE
) {
  if (balance_probabilities == FALSE) {
    .spatial_balance_measure_wrapper_equal("local2", sample, sample_size, spread_mat)
  } else {
    .spatial_balance_measure_wrapper_equal("local", sample, sample_size, spread_mat)
  }
}

#' @describeIn spatial_balance_measure Voronoi spatial balance
#' @export
spatial_balance_voronoi = function(sample, probabilities, spread_mat) {
  .spatial_balance_measure_wrapper("voronoi", sample, probabilities, spread_mat)
}

#' @describeIn spatial_balance_measure Voronoi spatial balance
#' @export
spatial_balance_voronoi_equal = function(sample, spread_mat, sample_size = length(sample)) {
  .spatial_balance_measure_wrapper_equal("voronoi", sample, sample_size, spread_mat)
}

#' @describeIn spatial_balance_measure Energy spatial balance
#' @export
spatial_balance_energy = function(sample, probabilities, spread_mat) {
  .spatial_balance_measure_wrapper("energy-distance", sample, probabilities, spread_mat)
}

#' @describeIn spatial_balance_measure Energy spatial balance
#' @export
spatial_balance_energy_equal = function(sample, spread_mat, sample_size = length(sample)) {
  .spatial_balance_measure_wrapper_equal("energy-distance", sample, sample_size, spread_mat)
}

#' @describeIn spatial_balance_measure Energy spatial balance
#' @export
spatial_balance_all = function(sample, probabilities, spread_mat) {
  rust_spatial_balance_measure_all(
    as.integer(sample),
    as.double(probabilities),
    as.matrix(spread_mat)
  )
}

#' @describeIn spatial_balance_measure Energy spatial balance
#' @export
spatial_balance_all_equal = function(sample, spread_mat, sample_size = length(sample)) {
  rust_spatial_balance_measure_all_equal(
    as.integer(sample),
    as.integer(sample_size),
    as.matrix(spread_mat)
  )
}

#' @describeIn spatial_balance_measure Balance deviation
#' @export
balance_deviation = function(sample, probabilities, spread_mat) {
  rust_balance_deviation(
    as.integer(sample),
    as.double(probabilities),
    as.matrix(spread_mat)
  )
}

#' Balanced sampling
#'
#' @name Balanced sampling
#' @rdname balanced_sampling
#' @description
#' Selects balanced samples with prescribed inclusion probabilities from finite populations.
#'
#' @details
#' For the `cube` method, a fixed sized sample is obtained if the first column of `balance_mat` is
#' the inclusion probabilities. For `cube_stratified`, the inclusion probabilities are inserted
#' automatically.
#'
#' @param probabilities A vector of inclusion probabilities.
#' @param balance_mat A matrix of balancing covariates.
#' @param strata An integer vector with stratum numbers for each unit.
#' @inheritDotParams .sampling_defaults -max_iter -bucket_size
#'
#' @returns A vector of sample indices.
#'
#' @references
#' Deville, J. C. and Tillé, Y. (2004).
#' Efficient balanced sampling: the cube method.
#' Biometrika, 91(4), 893-912.
#'
#' Chauvet, G. and Tillé, Y. (2006).
#' A fast algorithm for balanced sampling.
#' Computational Statistics, 21(1), 53-62.
#'
#' Chauvet, G. (2009).
#' Stratified balanced sampling.
#' Survey Methodology, 35, 115-119.
#'
#' @examples
#' set.seed(12345);
#' N = 1000;
#' n = 100;
#' prob = rep(n/N, N);
#' xb = matrix(c(prob, runif(N * 2)), ncol = 3);
#' strata = c(rep(1L, 100), rep(2L, 200), rep(3L, 300), rep(4L, 400));
#'
#' s = cube(prob, xb);
#' plot(xb[, 2], xb[, 3], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = cube_stratified(prob, xb[, -1], strata);
#' plot(xb[, 2], xb[, 3], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' \donttest{
#' # Respects inclusion probabilities
#' set.seed(12345);
#' prob = c(0.2, 0.25, 0.35, 0.4, 0.5, 0.5, 0.55, 0.65, 0.7, 0.9);
#' N = length(prob);
#' xb = matrix(c(prob, runif(N * 2)), ncol = 3);
#'
#' ep = rep(0L, N);
#' r = 10000L;
#'
#' for (i in seq_len(r)) {
#'   s = cube(prob, xb);
#'   ep[s] = ep[s] + 1L;
#' }
#'
#' print(ep / r - prob);
#' }
#'
NULL

.balanced_wrapper = function(method, probabilities, balance_mat, ...) {
  args = .sampling_defaults(...);
  rust_balanced(
    as.double(probabilities),
    as.matrix(balance_mat),
    args$eps,
    method
  )
}

#' @describeIn balanced_sampling The cube method
#' @export
cube = function(probabilities, balance_mat, ...) {
  .balanced_wrapper("cube", probabilities, balance_mat, ...)
}

#' @describeIn balanced_sampling The stratified cube method
#' @export
cube_stratified = function(probabilities, balance_mat, strata, ...) {
  args = .sampling_defaults(...);
  rust_balanced_stratified(
    as.numeric(probabilities),
    as.matrix(balance_mat),
    as.integer(strata),
    args$eps,
    "cube"
  )
}

#' Sampling defaults
#'
#' @param eps A small value used when comparing floats.
#' @param max_iter The maximum number of iterations used in iterative algorithms.
#' @param bucket_size The maximum size of the k-d-tree nodes. A higher value gives a slower
#'   k-d-tree, but is faster to create and takes up less memory.
#'
#' @returns A validated list of arguments used internally in sampling functions.
.sampling_defaults = function(
  eps = 1e-10,
  max_iter = 1000L,
  bucket_size = 50L
) {
  args = list(
    eps = as.double(eps),
    max_iter = as.integer(max_iter),
    bucket_size = as.integer(bucket_size)
  );

  if (!(length(args$eps) == 1 && 0.0 <= args$eps)) {
    warning("eps is negative ... using default");
    args$eps = 1e-12;
  }

  if (!(length(args$bucket_size) == 1 && 0 < args$bucket_size)) {
    warning("bucket_size is non-positive ... using default");
    args$bucket_size = 50L;
  }

  if (!(length(args$max_iter) == 1 && 0 < args$max_iter)) {
    warnings("max_iter is non-positive ... using default");
    args$max_iter = 1000L;
  }

  args
}

#' Transform a sample vector into an inclusion indicator vector
#'
#' @param sample A vector of sample indices.
#' @param population_size The total size of the population.
#'
#' @returns An inclusion indicator vector, i.e. a `population_size`-sized vector of 0/1.
#'
#' @examples
#' s = c(1, 2, 10);
#' si = sample_to_indicator(s, 10);
#'
#' @export
sample_to_indicator = function(sample, population_size) as.integer(seq_len(population_size) %in% sample)

#' Variance estimator for spatially balanced samples
#'
#' @description
#' Variance estimator of HT estimator of population total.
#'
#' @param values A vector of values of the variable of interest.
#' @param probabilities A vector of inclusion probabilities.
#' @param spread_mat A matrix of spreading covariates.
#' @param neighbours The number of neighbours to construct the means around.
#'
#' @returns A vector of sample indices.
#'
#' @references
#' Grafström, A., & Schelin, L. (2014).
#' How to select representative samples.
#' Scandinavian Journal of Statistics, 41(2), 277-290.
#'
#' @examples
#' set.seed(12345);
#' N = 1000;
#' n = 100;
#' prob = rep(n/N, N);
#' xs = matrix(runif(N * 2), ncol = 2);
#' y = runif(N);
#'
#' s = lpm_2(prob, xs);
#' local_mean_variance(y[s], prob[s], xs[s, ], 4);
#'
#' \donttest{
#' # Compare SRS, empirical
#' r = 1000L;
#' v = matrix(0.0, r, 3L);
#'
#' for (i in seq_len(r)) {
#'   s = lpm_2(prob, xs);
#'   v[i, 1] = local_mean_variance(y[s], prob[s], xs[s, ], 4);
#'   v[i, 2] = N^2 * sd(y[s]) / n;
#'   v[i, 3] = sum(y[s] / prob[s]);
#' }
#'
#' # Local mean variance, SRS variance, MSE
#' print(c(mean(v[, 1]), mean(v[, 2]), mean((v[, 3] - sum(y))^2)));
#' }
#'
#' @export
local_mean_variance = function(values, probabilities, spread_mat, neighbours = 4L) {
  neighbours = as.integer(neighbours);
  if (neighbours < 1L) {
    stop("neighbours needs to be positive");
  } else if (neighbours == 1L) {
    return (NaN);
  }

  rust_local_mean_variance(
    as.double(values),
    as.double(probabilities),
    as.matrix(spread_mat),
    neighbours
  )
}

#' Spatially balanced sampling
#'
#' @name Spatially balanced sampling
#' @rdname spatially_balanced_sampling
#' @description
#' Selects spatially balanced samples with prescribed inclusion probabilities from finite
#' populations.
#'
#' @details
#' `lpm_2_hierarchical` selects an initial sample using the LPM2 algorithm, and then splits this
#' sample into subsamples of given `sizes`, using successive, hierarchical selection with LPM2.
#' When using `lpm_2_hierarchical`, the inclusion probabilities must sum to an integer, and the
#' `sizes` vector (the subsamples) must sum to the same integer.
#'
#' @param probabilities A vector of inclusion probabilities.
#' @param spread_mat A matrix of spreading covariates.
#' @param sizes A vector of integers containing the sizes of the subsamples.
#' @inheritDotParams .sampling_defaults -max_iter
#'
#' @returns A vector of sample indices, or in the case of hierarchical sampling, a matrix where the
#' first column contains sample indices and the second column contains subsample indices (groups).
#'
#' @references
#' Deville, J.-C., &  Tillé, Y. (1998).
#' Unequal probability sampling without replacement through a splitting method.
#' Biometrika 85, 89-101.
#'
#' Grafström, A. (2012).
#' Spatially correlated Poisson sampling.
#' Journal of Statistical Planning and Inference, 142(1), 139-147.
#'
#' Grafström, A., Lundström, N.L.P. & Schelin, L. (2012).
#' Spatially balanced sampling through the Pivotal method.
#' Biometrics 68(2), 514-520.
#'
#' Lisic, J. J., & Cruze, N. B. (2016, June).
#' Local pivotal methods for large surveys.
#' In Proceedings of the Fifth International Conference on Establishment Surveys.
#'
#' Prentius, W. (2024).
#' Locally correlated Poisson sampling.
#' Environmetrics, 35(2), e2832.
#'
#' @examples
#' set.seed(12345);
#' N = 1000;
#' n = 100;
#' prob = rep(n/N, N);
#' xs = matrix(runif(N * 2), ncol = 2);
#' sizes = c(10L, 20L, 30L, 40L);
#'
#' s = lpm_1(prob, xs);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = lpm_1s(prob, xs);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = lpm_2(prob, xs);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = scps(prob, xs);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = lpm_2_hierarchical(prob, xs, sizes);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' \donttest{
#' s = lcps(prob, xs); # May have a long execution time
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' # Respects inclusion probabilities
#' set.seed(12345);
#' prob = c(0.2, 0.25, 0.35, 0.4, 0.5, 0.5, 0.55, 0.65, 0.7, 0.9);
#' N = length(prob);
#' xs = matrix(c(prob, runif(N * 2)), ncol = 3);
#'
#' ep = rep(0L, N);
#' r = 10000L;
#'
#' for (i in seq_len(r)) {
#'   s = lpm_2(prob, xs);
#'   ep[s] = ep[s] + 1L;
#' }
#'
#' print(ep / r - prob);
#' }
#'
NULL

.spatially_balanced_wrapper = function(method, probabilities, spread_mat, ...) {
  args = .sampling_defaults(...);

  rust_spatially_balanced(
    as.double(probabilities),
    as.matrix(spread_mat),
    args$eps,
    args$bucket_size,
    method
  )
}

#' @describeIn spatially_balanced_sampling Local pivotal method 1
#' @export
lpm_1 = function(probabilities, spread_mat, ...) {
  .spatially_balanced_wrapper("lpm_1", probabilities, spread_mat, ...)
}

#' @describeIn spatially_balanced_sampling Local pivotal method 1s
#' @export
lpm_1s = function(probabilities, spread_mat, ...) {
  .spatially_balanced_wrapper("lpm_1s", probabilities, spread_mat, ...)
}

#' @describeIn spatially_balanced_sampling Local pivotal method 2
#' @export
lpm_2 = function(probabilities, spread_mat, ...) {
  .spatially_balanced_wrapper("lpm_2", probabilities, spread_mat, ...)
}

#' @describeIn spatially_balanced_sampling Spatially correlated Poisson sampling
#' @export
scps = function(probabilities, spread_mat, ...) {
  .spatially_balanced_wrapper("scps", probabilities, spread_mat, ...)
}

#' @describeIn spatially_balanced_sampling Locally correlated Poisson sampling
#' @export
lcps = function(probabilities, spread_mat, ...) {
  .spatially_balanced_wrapper("lcps", probabilities, spread_mat, ...)
}

#' @describeIn spatially_balanced_sampling Hierarchical Local pivotal method 2
#' @export
lpm_2_hierarchical = function(probabilities, spread_mat, sizes, ...) {
  args = .sampling_defaults(...);
  rust_spatially_balanced_hierarchical(
    as.double(probabilities),
    as.matrix(spread_mat),
    as.integer(sizes),
    args$eps,
    args$bucket_size,
    "lpm_2"
  )
}

#' Distributionally balanced designs
#'
#' @name Distributionally balanced designs
#' @rdname distributional_designs
#' @description
#' Construct distributionally balanced sampling designs.
#'
#' @param sample_size The size of the desired sample.
#' @param spread_mat A matrix of spreading covariates.
#' @inheritDotParams .dbd_defaults
#'
#' @returns A `dbd` object containing all possible samples.
#'
#' @details
#' Use the [draw.dbd()] method to draw or select a single sample from the design.
#'
#' @references
#' Grafström, A., & Prentius, W. (2026).
#' Distributionally balanced sampling designs.
#' arXiv preprint arXiv:2603.11916.
#'
#' Grafström, A., & Prentius, W. (2026).
#' Distributionally balanced sampling designs via minimum tactical configurations.
#' arXiv preprint arXiv:2603.24439.
#'
#' @examples
#' set.seed(12345);
#' N = 1000L;
#' n = 100L;
#' prob = rep(n / N, N);
#' xs = matrix(runif(N * 2), ncol = 2);
#'
#' # Construct circular dbd design
#' design = dbd_circular(
#'   n,
#'   xs,
#'   annealing_temp = 0.1,
#'   annealing_cooling = 0.999,
#'   max_iter = 1e5L
#' );
#' s = draw(design); # Draw sample from design
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' # Construct dbd design using tactical configuration
#' design = dbd_tc(
#'   n,
#'   xs,
#'   annealing_temp = 0.1,
#'   annealing_cooling = 0.999,
#'   max_iter = 1e5L
#' );
#' s = draw(design); # Draw sample from design
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
NULL

#' Distributionally balanced designs defaults
#'
#' @param annealing_temp The initial temperature to use in simulated annealing
#' @param annealing_cooling The annealing_cooling rate tu use in simulated annealing
#' @param spatial_init If `TRUE` a spatial initialization strategy is used
#' @inheritDotParams .sampling_defaults -bucket_size
#'
#' @returns A validated list of arguments used internally in dbd functions.
.dbd_defaults = function(
  annealing_temp = 0.1,
  annealing_cooling = 0.999,
  spatial_init = FALSE,
  ...
) {
  args = c(
    .sampling_defaults(...),
    list(
      annealing_temp    = as.double(annealing_temp),
      annealing_cooling = as.double(annealing_cooling),
      spatial_init      = as.logical(spatial_init)
    )
  );

  if (!(0.0 < args$annealing_temp)) {
    warning("annealing_temp is non-positive ... using default");
    args$annealing_temp = 0.1;
  }

  if (!(0.0 < args$annealing_cooling && args$annealing_cooling < 1.0)) {
    warning("annealing_cooling is not in (0.0, 1.0) ... using default");
    args$annealing_cooling = 0.999;
  }

  args
}

#' Distributionally balanced designs wrapper
#' Calls [.dbd_defaults()]
#'
#' @param method The method name as defined by the rust wrapper
#' @param sample_size The integer-valued desired sample size
#' @param spread_mat A matrix of auxiliary variables
#'
#' @noRd
.distributionally_balanced_wrapper = function(
  method,
  sample_size,
  spread_mat,
  ...
) {
  args = .dbd_defaults(...);

  spread_mat = as.matrix(spread_mat);
  sample_size = as.integer(sample_size);

  if (!(0L < sample_size && sample_size < nrow(spread_mat))) {
    stop("sample_size must be positive and less than the population size");
  }

  s = rust_distributionally_balanced_design(
    sample_size,
    spread_mat,
    args$annealing_temp,
    args$annealing_cooling,
    args$max_iter,
    args$spatial_init,
    method
  );

  if (method == "dbd_circular") {
    attr(s, "number_of_samples") = length(s);
  } else if (method == "dbd_tc") {
    s = matrix(s, sample_size);
    attr(s, "number_of_samples") = ncol(s);
  } else {
    # Should be unreachable
    stop("method not supported");
  }

  class(s) = c("dbd", class(s));
  attr(s, "method") = method;
  attr(s, "sample_size") = sample_size;

  for (arg in names(args)) {
    attr(s, arg) = args[[arg]];
  }

  s
}

#' @describeIn distributional_designs Distributionally Balanced Sampling as circular sequence
#' @export
dbd_circular = function(
  sample_size,
  spread_mat,
  ...
) {
  .distributionally_balanced_wrapper("dbd_circular", sample_size, spread_mat, ...)
}

#' @describeIn distributional_designs Distributionally Balanced Sampling using Tactical Configuration
#' @export
dbd_tc = function(
  sample_size,
  spread_mat,
  ...
) {
  .distributionally_balanced_wrapper("dbd_tc", sample_size, spread_mat, ...)
}

#' Wrapper
#' @noRd
.distributionally_balanced_iter_wrapper = function(
  method,
  sample_size,
  spread_mat,
  iter_by = 100L,
  ...
) {
  args = .dbd_defaults(...);

  iter_by = as.integer(iter_by);

  if (!(length(iter_by) == 1 && 0 < iter_by && iter_by <= args$max_iter)) {
    warnings("iter_by is not within within (0, max_iter) ... using default");
    iter_by = min(100L, args$max_iter);
  }


  s = rust_distributionally_balanced_design_iter(
    sample_size,
    as.matrix(spread_mat),
    args$annealing_temp,
    args$annealing_cooling,
    args$spatial_init,
    args$max_iter,
    iter_by,
    method
  );

  matrix(s, ncol = 2, byrow = TRUE)
}

#' Iterative evaluation of distributional designs
#'
#' @name Iterative evaluation of distributional designs
#' @rdname distributional_designs_iter
#' @description
#' Evaluates the energy distance along a simulated annealing scheme.
#'
#' @param sample_size The size of the desired sample.
#' @param spread_mat A matrix of spreading covariates.
#' @param ... Additional arguments, see [.dbd_defaults()].
#'   Note:
#'   - `max_iter` determines the number of iterations to run;
#'   - `iter_by` determines the reporting interval.
#'
#' @returns A matrix with the average energies and the standard deviations of the energies as
#' columns, per the reporting intervals.
#'
#' @examples
#' set.seed(12345);
#' N = 1000L;
#' n = 100L;
#' prob = rep(n / N, N);
#' xs = matrix(runif(N * 2), ncol = 2);
#'
#' # Report mean and sd energy at every 1e4L iteration
#' dbd_circular_iter(
#'   n,
#'   xs,
#'   annealing_temp = 0.1,
#'   annealing_cooling = 0.999,
#'   max_iter = 1e5L,
#'   iter_by = 1e4L
#' );
#'
#' # Report mean and sd energy at every 1e4L iteration
#' dbd_tc_iter(
#'   n,
#'   xs,
#'   annealing_temp = 0.1,
#'   annealing_cooling = 0.999,
#'   max_iter = 1e5L,
#'   iter_by = 1e4L
#' );
#'
#' @export
dbd_circular_iter = function(
  sample_size,
  spread_mat,
  ...
) {
  .distributionally_balanced_iter_wrapper("dbd_circular", sample_size, spread_mat, ...)
}

#' @describeIn distributional_designs_iter Iterative evolution of distributional design using
#' Tactical Configuration
#' @export
dbd_tc_iter = function(
  sample_size,
  spread_mat,
  ...
) {
  .distributionally_balanced_iter_wrapper("dbd_tc", sample_size, spread_mat, ...)
}

#' @keywords internal
#' @docType package
#' @name rsamplr
#'
#' @description
#' Select probability samples in multi-dimensional spaces with any prescribed inclusion
#' probabilities using fast rust implementations.
#'
#' You can access the project website at
#' <https://envisim.se>.
#'
#' @author Wilmer Prentius \email{wilmer.prentius@gmail.com}, Anton Grafström.
#'
#' @references
#' Benedetti, R., Piersimoni, F., & Postiglione, P. (2017).
#' Spatially balanced sampling: a review and a reappraisal.
#' International Statistical Review, 85(3), 439-454.
#'
#' Friedman, J. H., Bentley, J. L., & Finkel, R. A. (1977).
#' An algorithm for finding best matches in logarithmic expected time.
#' ACM Transactions on Mathematical Software (TOMS), 3(3), 209-226.
#'
#' Maneewongvatana, S., & Mount, D. M. (1999).
#' It’s okay to be skinny, if your friends are fat.
#' In Center for geometric computing 4th annual workshop on computational geometry (Vol. 2, pp. 1-8).
#'
#' Särndal, C. E., Swensson, B., & Wretman, J. (2003).
#' Model assisted survey sampling.
#' Springer Science & Business Media.
#'
#' Tillé, Y. (2006).
#' Sampling algorithms.
#' New York, NY: Springer New York.
#'
"_PACKAGE"
NULL

#' Unequal probability sampling
#'
#' @name Unequal probability sampling
#' @rdname unequal_probability_sampling
#' @description
#' Selects samples with prescribed inclusion probabilities from finite populations.
#'
#' @param probabilities A vector of inclusion probabilities.
#' @param sample_size The wanted sample size
#' @inheritDotParams .sampling_defaults -bucket_size
#'
#' @returns A vector of sample indices.
#'
#' @details
#' `sampford` and `conditional_poisson` may return an error if a solution is not found within
#' `max_iter`.
#'
#' @references
#' Bondesson, L., & Thorburn, D. (2008).
#' A list sequential sampling method suitable for real‐time sampling.
#' Scandinavian Journal of Statistics, 35(3), 466-483.
#'
#' Brewer, K. E. (1975).
#' A Simple Procedure for Sampling pi-ps wor.
#' Australian Journal of Statistics, 17(3), 166-172.
#'
#' Chauvet, G. (2012).
#' On a characterization of ordered pivotal sampling.
#' Bernoulli, 18(4), 1320-1340.
#'
#' Deville, J.-C., &  Tillé, Y. (1998).
#' Unequal probability sampling without replacement through a splitting method.
#' Biometrika 85, 89-101.
#'
#' Grafström, A. (2009).
#' Non-rejective implementations of the Sampford sampling design.
#' Journal of Statistical Planning and Inference, 139(6), 2111-2114.
#'
#' Rosén, B. (1997).
#' On sampling with probability proportional to size.
#' Journal of statistical planning and inference, 62(2), 159-191.
#'
#' Sampford, M. R. (1967).
#' On sampling without replacement with unequal probabilities of selection.
#' Biometrika, 54(3-4), 499-513.
#'
#' @examples
#' set.seed(12345);
#' N = 1000;
#' n = 100;
#' prob = rep(n/N, N);
#' xs = matrix(runif(N * 2), ncol = 2);
#'
#' s = rpm(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = spm(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = cps(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = poisson(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = brewer(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = pareto(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = systematic(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = systematic_random_order(prob);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' # Conditional poisson and sampford are not guaranteed to find a solution
#' prob2 = rep(0.5, 10L);
#' s = conditional_poisson(prob2, 5L, max_iter = 10000L);
#' plot(xs[1:10, 1], xs[1:10, 2], pch = ifelse(sample_to_indicator(s, 10L), 19, 1));
#'
#' s = sampford(prob2, max_iter = 10000L);
#' plot(xs[1:10, 1], xs[1:10, 2], pch = ifelse(sample_to_indicator(s, 10L), 19, 1));
#'
#' \donttest{
#' # Respects inclusion probabilities
#' set.seed(12345);
#' prob = c(0.2, 0.25, 0.35, 0.4, 0.5, 0.5, 0.55, 0.65, 0.7, 0.9);
#' N = length(prob);
#'
#' ep = rep(0L, N);
#' r = 10000L;
#'
#' for (i in seq_len(r)) {
#'   s = poisson(prob);
#'   ep[s] = ep[s] + 1L;
#' }
#'
#' print(ep / r - prob);
#' }
#'
NULL

.unequal_wrapper = function(method, probabilities, sample_size = 1L, ...) {
  args = .sampling_defaults(...);
  rust_unequal(
    as.double(probabilities),
    args$eps,
    method,
    args$max_iter,
    as.integer(sample_size)
  )
}

#' @describeIn unequal_probability_sampling Random pivotal method
#' @export
rpm = function(probabilities, ...) {
  .unequal_wrapper("rpm", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Sequential pivotal method
#' @export
spm = function(probabilities, ...) {
  .unequal_wrapper("spm", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Correlated Poisson sampling
#' @export
cps = function(probabilities, ...) {
  .unequal_wrapper("cps", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Poisson sampling
#' @export
poisson = function(probabilities, ...) {
  .unequal_wrapper("poisson", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Conditional Poisson sampling
#' @export
conditional_poisson = function(probabilities, sample_size, ...) {
  .unequal_wrapper("poisson", probabilities, sample_size, ...)
}

#' @describeIn unequal_probability_sampling Systematic sampling
#' @export
systematic = function(probabilities, ...) {
  .unequal_wrapper("systematic", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Systematic sampling with random order
#' @export
systematic_random_order = function(probabilities, ...) {
  .unequal_wrapper("systematic_random_order", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Brewer sampling
#' @export
brewer = function(probabilities, ...) {
  .unequal_wrapper("brewer", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Pareto sampling
#' @export
pareto = function(probabilities, ...) {
  .unequal_wrapper("pareto", probabilities, ...)
}

#' @describeIn unequal_probability_sampling Sampford sampling
#' @export
sampford = function(probabilities, ...) {
  .unequal_wrapper("sampford", probabilities, ...)
}

#' Doubly balanced sampling
#'
#' @name Doubly balanced sampling
#' @rdname doubly_balanced_sampling
#' @description
#' Selects doubly balanced samples with prescribed inclusion probabilities from finite populations.
#'
#' @details
#' For the `local_cube` method, a fixed sized sample is obtained if the first column of
#' `balance_mat` is the inclusion probabilities. For `local_cube_stratified`, the inclusion
#' probabilities are inserted automatically.
#'
#' @param probabilities A vector of inclusion probabilities.
#' @param spread_mat A matrix of spreading covariates.
#' @param balance_mat A matrix of balancing covariates.
#' @param strata An integer vector with stratum numbers for each unit.
#' @inheritDotParams .sampling_defaults -max_iter
#'
#' @returns A vector of sample indices.
#'
#' @references
#' Deville, J. C. and Tillé, Y. (2004).
#' Efficient balanced sampling: the cube method.
#' Biometrika, 91(4), 893-912.
#'
#' Chauvet, G. and Tillé, Y. (2006).
#' A fast algorithm for balanced sampling.
#' Computational Statistics, 21(1), 53-62.
#'
#' Chauvet, G. (2009).
#' Stratified balanced sampling.
#' Survey Methodology, 35, 115-119.
#'
#' Grafström, A. and Tillé, Y. (2013).
#' Doubly balanced spatial sampling with spreading and restitution of auxiliary totals.
#' Environmetrics, 24(2), 120-131
#'
#' @examples
#' set.seed(12345);
#' N = 1000;
#' n = 100;
#' prob = rep(n/N, N);
#' xb = matrix(c(prob, runif(N * 2)), ncol = 3);
#' xs = matrix(runif(N * 2), ncol = 2);
#' strata = c(rep(1L, 100), rep(2L, 200), rep(3L, 300), rep(4L, 400));
#'
#' s = local_cube(prob, xs, xb);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' s = local_cube_stratified(prob, xs, xb[, -1], strata);
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#' \donttest{
#' # Respects inclusion probabilities
#' set.seed(12345);
#' prob = c(0.2, 0.25, 0.35, 0.4, 0.5, 0.5, 0.55, 0.65, 0.7, 0.9);
#' N = length(prob);
#' xb = matrix(c(prob, runif(N * 2)), ncol = 3);
#' xs = matrix(runif(N * 2), ncol = 2);
#'
#' ep = rep(0L, N);
#' r = 10000L;
#'
#' for (i in seq_len(r)) {
#'   s = local_cube(prob, xs, xb);
#'   ep[s] = ep[s] + 1L;
#' }
#'
#' print(ep / r - prob);
#' }
#'
NULL

.doubly_balanced_wrapper = function(method, probabilities, spread_mat, balance_mat, ...) {
  args = .sampling_defaults(...);
  rust_doubly_balanced(
    as.double(probabilities),
    as.matrix(spread_mat),
    as.matrix(balance_mat),
    args$eps,
    args$bucket_size,
    method
  )
}

#' @describeIn doubly_balanced_sampling The local cube method
#' @export
local_cube = function(probabilities, spread_mat, balance_mat, ...) {
  .doubly_balanced_wrapper("local_cube", probabilities, spread_mat, balance_mat, ...)
}

#' @describeIn doubly_balanced_sampling The stratified local cube method
#' @export
local_cube_stratified = function(probabilities, spread_mat, balance_mat, strata, ...) {
  args = .sampling_defaults(...);
  rust_doubly_balanced_stratified(
    as.double(probabilities),
    as.matrix(spread_mat),
    as.matrix(balance_mat),
    as.integer(strata),
    args$eps,
    args$bucket_size,
    "local_cube"
  );
}

#' Draw from a sampling design
#'
#' @param design A design object.
#' @param ... Additional arguments passed to methods.
#'
#' @describeIn draw.dbd draw generic
#' @export
draw = function(design, ...) {
  UseMethod("draw");
}

#' Draw a sample from a distributionally balanced design
#'
#' @param design A `dbd` design object.
#' @param ... Additional arguments passed to methods. `draw.dbd` accepts `sample_id` (integer). If
#' `sample_id` is `0L`, returns a random sample (default).
#'
#' @returns a vector of sample indices.
#'
#' @examples
#' set.seed(12345);
#' N = 1000L;
#' n = 100L;
#' prob = rep(n / N, N);
#' xs = matrix(runif(N * 2), ncol = 2);
#'
#' # Construct dbd design using tactical configuration
#' design = dbd_tc(
#'   n,
#'   xs,
#'   annealing_temp = 0.1,
#'   annealing_cooling = 0.999,
#'   max_iter = 1e5L
#' );
#' s = draw(design); # Draw sample from design
#' plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));
#'
#'
#' @export
draw.dbd = function(design, ...) {
  args = list(...);
  method = attr(design, "method");
  no_samples = attr(design, "number_of_samples");

  sample_id = as.integer(args$sample_id);
  if (length(sample_id) == 0 || !(0L < sample_id && sample_id <= no_samples)) {
    sample_id = sample(no_samples, 1L);
  }

  if (method == "dbd_circular") {
    pop_size = length(design);
    sample_size = attr(design, "sample_size");

    if (sample_id <= pop_size - sample_size + 1) {
      s = design[sample_id:(sample_id + sample_size - 1)];
      sort(s);
      return(s);
    } else {
      s = design[c(1:(sample_size - pop_size + sample_id - 1), sample_id:pop_size)];
      sort(s);
      return(s);
    }
  } else if (method == "dbd_tc") {
    return(design[, sample_id]);
  } else {
    # Should be unreachable
    stop("method not supported");
  }
}

# Generated by savvy: do not edit by hand
#
# Note:
#   This wrapper file is named as `000-wrappers.R` so that this file is loaded
#   first, which allows users to override the functions defined here (e.g., a
#   print() method for an enum).

#' @useDynLib rsamplr, .registration = TRUE
#' @keywords internal
NULL

# Check class and extract the external pointer embedded in the environment
.savvy_extract_ptr <- function(e, class) {
  if(is.null(e)) {
    return(NULL)
  }

  if(inherits(e, class)) {
    e$.ptr
  } else {
    msg <- paste0("Expected ", class, ", got ", class(e)[1])
    stop(msg, call. = FALSE)
  }
}

# Prohibit modifying environments

#' @export
`$<-.savvy_rsamplr__sealed` <- function(x, name, value) {
  class <- gsub("__bundle$", "", class(x)[1])
  stop(class, " cannot be modified", call. = FALSE)
}

#' @export
`[[<-.savvy_rsamplr__sealed` <- function(x, i, value) {
  class <- gsub("__bundle$", "", class(x)[1])
  stop(class, " cannot be modified", call. = FALSE)
}


`rust_balance_deviation` <- function(`r_sample`, `r_prob`, `r_data`) {
  .Call(savvy_rust_balance_deviation__impl, `r_sample`, `r_prob`, `r_data`)
}


`rust_balanced` <- function(`r_prob`, `r_bal_data`, `r_eps`, `r_method`) {
  .Call(savvy_rust_balanced__impl, `r_prob`, `r_bal_data`, `r_eps`, `r_method`)
}


`rust_balanced_stratified` <- function(`r_prob`, `r_bal_data`, `r_strata`, `r_eps`, `r_method`) {
  .Call(savvy_rust_balanced_stratified__impl, `r_prob`, `r_bal_data`, `r_strata`, `r_eps`, `r_method`)
}


`rust_distributionally_balanced_design` <- function(`r_sample_size`, `r_data`, `r_temp`, `r_cooling`, `r_iter`, `r_spatial_init`, `r_method`) {
  .Call(savvy_rust_distributionally_balanced_design__impl, `r_sample_size`, `r_data`, `r_temp`, `r_cooling`, `r_iter`, `r_spatial_init`, `r_method`)
}


`rust_distributionally_balanced_design_iter` <- function(`r_sample_size`, `r_data`, `r_temp`, `r_cooling`, `r_spatial_init`, `r_iter_to`, `r_iter_by`, `r_method`) {
  .Call(savvy_rust_distributionally_balanced_design_iter__impl, `r_sample_size`, `r_data`, `r_temp`, `r_cooling`, `r_spatial_init`, `r_iter_to`, `r_iter_by`, `r_method`)
}


`rust_doubly_balanced` <- function(`r_prob`, `r_data`, `r_bal_data`, `r_eps`, `r_bucket_size`, `r_method`) {
  .Call(savvy_rust_doubly_balanced__impl, `r_prob`, `r_data`, `r_bal_data`, `r_eps`, `r_bucket_size`, `r_method`)
}


`rust_doubly_balanced_stratified` <- function(`r_prob`, `r_data`, `r_bal_data`, `r_strata`, `r_eps`, `r_bucket_size`, `r_method`) {
  .Call(savvy_rust_doubly_balanced_stratified__impl, `r_prob`, `r_data`, `r_bal_data`, `r_strata`, `r_eps`, `r_bucket_size`, `r_method`)
}


`rust_local_mean_variance` <- function(`r_values`, `r_prob`, `r_data`, `r_neighbours`) {
  .Call(savvy_rust_local_mean_variance__impl, `r_values`, `r_prob`, `r_data`, `r_neighbours`)
}


`rust_pips_from_values` <- function(`r_values`, `r_sample_size`) {
  .Call(savvy_rust_pips_from_values__impl, `r_values`, `r_sample_size`)
}


`rust_spatial_balance_measure` <- function(`r_sample`, `r_prob`, `r_data`, `r_method`) {
  .Call(savvy_rust_spatial_balance_measure__impl, `r_sample`, `r_prob`, `r_data`, `r_method`)
}


`rust_spatial_balance_measure_all` <- function(`r_sample`, `r_prob`, `r_data`) {
  .Call(savvy_rust_spatial_balance_measure_all__impl, `r_sample`, `r_prob`, `r_data`)
}


`rust_spatial_balance_measure_all_equal` <- function(`r_sample`, `r_sample_size`, `r_data`) {
  .Call(savvy_rust_spatial_balance_measure_all_equal__impl, `r_sample`, `r_sample_size`, `r_data`)
}


`rust_spatial_balance_measure_equal` <- function(`r_sample`, `r_sample_size`, `r_data`, `r_method`) {
  .Call(savvy_rust_spatial_balance_measure_equal__impl, `r_sample`, `r_sample_size`, `r_data`, `r_method`)
}


`rust_spatially_balanced` <- function(`r_prob`, `r_data`, `r_eps`, `r_bucket_size`, `r_method`) {
  .Call(savvy_rust_spatially_balanced__impl, `r_prob`, `r_data`, `r_eps`, `r_bucket_size`, `r_method`)
}


`rust_spatially_balanced_hierarchical` <- function(`r_prob`, `r_data`, `r_sizes`, `r_eps`, `r_bucket_size`, `r_method`) {
  .Call(savvy_rust_spatially_balanced_hierarchical__impl, `r_prob`, `r_data`, `r_sizes`, `r_eps`, `r_bucket_size`, `r_method`)
}


`rust_unequal` <- function(`r_prob`, `r_eps`, `r_method`, `r_max_iter`, `r_sample_size`) {
  .Call(savvy_rust_unequal__impl, `r_prob`, `r_eps`, `r_method`, `r_max_iter`, `r_sample_size`)
}

# Authors of vendored cargo crates
- cc 1.2.58: Alex Crichton
- envisim_estimate 0.4.0: Wilmer Prentius
- envisim_samplr 0.5.0: Wilmer Prentius
- envisim_utils 0.4.0: Wilmer Prentius
- proc-macro2 1.0.106: David Tolnay, Alex Crichton
- quote 1.0.45: David Tolnay
- rustc-hash 2.1.2: The Rust Project Developers
- savvy 0.9.2: Hiroaki Yutani
- savvy-bindgen 0.9.2: Hiroaki Yutani
- savvy-ffi 0.9.2: Hiroaki Yutani
- savvy-macro 0.9.2: Hiroaki Yutani
- shlex 1.3.0: comex, Fenhl, Adrian Taylor, Alex Touchet, Daniel Parks, Garrett Berg
- syn 2.0.117: David Tolnay
- unicode-ident 1.0.24: David Tolnay

(This file was auto-generated from 'cargo metadata' on 2026-03-30)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatially_balanced_sampling.R
\name{Spatially balanced sampling}
\alias{Spatially balanced sampling}
\alias{lpm_1}
\alias{lpm_1s}
\alias{lpm_2}
\alias{scps}
\alias{lcps}
\alias{lpm_2_hierarchical}
\title{Spatially balanced sampling}
\usage{
lpm_1(probabilities, spread_mat, ...)

lpm_1s(probabilities, spread_mat, ...)

lpm_2(probabilities, spread_mat, ...)

scps(probabilities, spread_mat, ...)

lcps(probabilities, spread_mat, ...)

lpm_2_hierarchical(probabilities, spread_mat, sizes, ...)
}
\arguments{
\item{probabilities}{A vector of inclusion probabilities.}

\item{spread_mat}{A matrix of spreading covariates.}

\item{...}{
  Arguments passed on to \code{\link[=.sampling_defaults]{.sampling_defaults}}
  \describe{
    \item{\code{eps}}{A small value used when comparing floats.}
    \item{\code{bucket_size}}{The maximum size of the k-d-tree nodes. A higher value gives a slower
k-d-tree, but is faster to create and takes up less memory.}
  }}

\item{sizes}{A vector of integers containing the sizes of the subsamples.}
}
\value{
A vector of sample indices, or in the case of hierarchical sampling, a matrix where the
first column contains sample indices and the second column contains subsample indices (groups).
}
\description{
Selects spatially balanced samples with prescribed inclusion probabilities from finite
populations.
}
\details{
\code{lpm_2_hierarchical} selects an initial sample using the LPM2 algorithm, and then splits this
sample into subsamples of given \code{sizes}, using successive, hierarchical selection with LPM2.
When using \code{lpm_2_hierarchical}, the inclusion probabilities must sum to an integer, and the
\code{sizes} vector (the subsamples) must sum to the same integer.
}
\section{Functions}{
\itemize{
\item \code{lpm_1()}: Local pivotal method 1

\item \code{lpm_1s()}: Local pivotal method 1s

\item \code{lpm_2()}: Local pivotal method 2

\item \code{scps()}: Spatially correlated Poisson sampling

\item \code{lcps()}: Locally correlated Poisson sampling

\item \code{lpm_2_hierarchical()}: Hierarchical Local pivotal method 2

}}
\examples{
set.seed(12345);
N = 1000;
n = 100;
prob = rep(n/N, N);
xs = matrix(runif(N * 2), ncol = 2);
sizes = c(10L, 20L, 30L, 40L);

s = lpm_1(prob, xs);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = lpm_1s(prob, xs);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = lpm_2(prob, xs);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = scps(prob, xs);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = lpm_2_hierarchical(prob, xs, sizes);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

\donttest{
s = lcps(prob, xs); # May have a long execution time
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

# Respects inclusion probabilities
set.seed(12345);
prob = c(0.2, 0.25, 0.35, 0.4, 0.5, 0.5, 0.55, 0.65, 0.7, 0.9);
N = length(prob);
xs = matrix(c(prob, runif(N * 2)), ncol = 3);

ep = rep(0L, N);
r = 10000L;

for (i in seq_len(r)) {
  s = lpm_2(prob, xs);
  ep[s] = ep[s] + 1L;
}

print(ep / r - prob);
}

}
\references{
Deville, J.-C., &  Tillé, Y. (1998).
Unequal probability sampling without replacement through a splitting method.
Biometrika 85, 89-101.

Grafström, A. (2012).
Spatially correlated Poisson sampling.
Journal of Statistical Planning and Inference, 142(1), 139-147.

Grafström, A., Lundström, N.L.P. & Schelin, L. (2012).
Spatially balanced sampling through the Pivotal method.
Biometrics 68(2), 514-520.

Lisic, J. J., & Cruze, N. B. (2016, June).
Local pivotal methods for large surveys.
In Proceedings of the Fifth International Conference on Establishment Surveys.

Prentius, W. (2024).
Locally correlated Poisson sampling.
Environmetrics, 35(2), e2832.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributional_designs.R
\name{Iterative evaluation of distributional designs}
\alias{Iterative evaluation of distributional designs}
\alias{dbd_circular_iter}
\alias{dbd_tc_iter}
\title{Iterative evaluation of distributional designs}
\usage{
dbd_circular_iter(sample_size, spread_mat, ...)

dbd_tc_iter(sample_size, spread_mat, ...)
}
\arguments{
\item{sample_size}{The size of the desired sample.}

\item{spread_mat}{A matrix of spreading covariates.}

\item{...}{Additional arguments, see \code{\link[=.dbd_defaults]{.dbd_defaults()}}.
Note:
\itemize{
\item \code{max_iter} determines the number of iterations to run;
\item \code{iter_by} determines the reporting interval.
}}
}
\value{
A matrix with the average energies and the standard deviations of the energies as
columns, per the reporting intervals.
}
\description{
Evaluates the energy distance along a simulated annealing scheme.
}
\section{Functions}{
\itemize{
\item \code{dbd_tc_iter()}: Iterative evolution of distributional design using
Tactical Configuration

}}
\examples{
set.seed(12345);
N = 1000L;
n = 100L;
prob = rep(n / N, N);
xs = matrix(runif(N * 2), ncol = 2);

# Report mean and sd energy at every 1e4L iteration
dbd_circular_iter(
  n,
  xs,
  annealing_temp = 0.1,
  annealing_cooling = 0.999,
  max_iter = 1e5L,
  iter_by = 1e4L
);

# Report mean and sd energy at every 1e4L iteration
dbd_tc_iter(
  n,
  xs,
  annealing_temp = 0.1,
  annealing_cooling = 0.999,
  max_iter = 1e5L,
  iter_by = 1e4L
);

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/draw.R
\name{draw}
\alias{draw}
\alias{draw.dbd}
\title{Draw from a sampling design}
\usage{
draw(design, ...)

\method{draw}{dbd}(design, ...)
}
\arguments{
\item{design}{A \code{dbd} design object.}

\item{...}{Additional arguments passed to methods. \code{draw.dbd} accepts \code{sample_id} (integer). If
\code{sample_id} is \code{0L}, returns a random sample (default).}
}
\value{
a vector of sample indices.
}
\description{
Draw from a sampling design

Draw a sample from a distributionally balanced design
}
\section{Functions}{
\itemize{
\item \code{draw()}: draw generic

}}
\examples{
set.seed(12345);
N = 1000L;
n = 100L;
prob = rep(n / N, N);
xs = matrix(runif(N * 2), ncol = 2);

# Construct dbd design using tactical configuration
design = dbd_tc(
  n,
  xs,
  annealing_temp = 0.1,
  annealing_cooling = 0.999,
  max_iter = 1e5L
);
s = draw(design); # Draw sample from design
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsamplr-package.R
\docType{package}
\name{rsamplr}
\alias{rsamplr-package}
\alias{rsamplr}
\title{rsamplr: Sampling Algorithms and Spatially Balanced Sampling}
\description{
Select probability samples in multi-dimensional spaces with any prescribed inclusion
probabilities using fast rust implementations.

You can access the project website at
\url{https://envisim.se}.
}
\references{
Benedetti, R., Piersimoni, F., & Postiglione, P. (2017).
Spatially balanced sampling: a review and a reappraisal.
International Statistical Review, 85(3), 439-454.

Friedman, J. H., Bentley, J. L., & Finkel, R. A. (1977).
An algorithm for finding best matches in logarithmic expected time.
ACM Transactions on Mathematical Software (TOMS), 3(3), 209-226.

Maneewongvatana, S., & Mount, D. M. (1999).
It’s okay to be skinny, if your friends are fat.
In Center for geometric computing 4th annual workshop on computational geometry (Vol. 2, pp. 1-8).

Särndal, C. E., Swensson, B., & Wretman, J. (2003).
Model assisted survey sampling.
Springer Science & Business Media.

Tillé, Y. (2006).
Sampling algorithms.
New York, NY: Springer New York.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://www.envisim.se/}
  \item \url{https://github.com/envisim/rust-samplr/}
  \item Report bugs at \url{https://github.com/envisim/rust-samplr/issues}
}

}
\author{
Wilmer Prentius \email{wilmer.prentius@gmail.com}, Anton Grafström.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local_mean_variance.R
\name{local_mean_variance}
\alias{local_mean_variance}
\title{Variance estimator for spatially balanced samples}
\usage{
local_mean_variance(values, probabilities, spread_mat, neighbours = 4L)
}
\arguments{
\item{values}{A vector of values of the variable of interest.}

\item{probabilities}{A vector of inclusion probabilities.}

\item{spread_mat}{A matrix of spreading covariates.}

\item{neighbours}{The number of neighbours to construct the means around.}
}
\value{
A vector of sample indices.
}
\description{
Variance estimator of HT estimator of population total.
}
\examples{
set.seed(12345);
N = 1000;
n = 100;
prob = rep(n/N, N);
xs = matrix(runif(N * 2), ncol = 2);
y = runif(N);

s = lpm_2(prob, xs);
local_mean_variance(y[s], prob[s], xs[s, ], 4);

\donttest{
# Compare SRS, empirical
r = 1000L;
v = matrix(0.0, r, 3L);

for (i in seq_len(r)) {
  s = lpm_2(prob, xs);
  v[i, 1] = local_mean_variance(y[s], prob[s], xs[s, ], 4);
  v[i, 2] = N^2 * sd(y[s]) / n;
  v[i, 3] = sum(y[s] / prob[s]);
}

# Local mean variance, SRS variance, MSE
print(c(mean(v[, 1]), mean(v[, 2]), mean((v[, 3] - sum(y))^2)));
}

}
\references{
Grafström, A., & Schelin, L. (2014).
How to select representative samples.
Scandinavian Journal of Statistics, 41(2), 277-290.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributional_designs.R
\name{Distributionally balanced designs}
\alias{Distributionally balanced designs}
\alias{dbd_circular}
\alias{dbd_tc}
\title{Distributionally balanced designs}
\usage{
dbd_circular(sample_size, spread_mat, ...)

dbd_tc(sample_size, spread_mat, ...)
}
\arguments{
\item{sample_size}{The size of the desired sample.}

\item{spread_mat}{A matrix of spreading covariates.}

\item{...}{
  Arguments passed on to \code{\link[=.dbd_defaults]{.dbd_defaults}}
  \describe{
    \item{\code{annealing_temp}}{The initial temperature to use in simulated annealing}
    \item{\code{annealing_cooling}}{The annealing_cooling rate tu use in simulated annealing}
    \item{\code{spatial_init}}{If \code{TRUE} a spatial initialization strategy is used}
  }}
}
\value{
A \code{dbd} object containing all possible samples.
}
\description{
Construct distributionally balanced sampling designs.
}
\details{
Use the \code{\link[=draw.dbd]{draw.dbd()}} method to draw or select a single sample from the design.
}
\section{Functions}{
\itemize{
\item \code{dbd_circular()}: Distributionally Balanced Sampling as circular sequence

\item \code{dbd_tc()}: Distributionally Balanced Sampling using Tactical Configuration

}}
\examples{
set.seed(12345);
N = 1000L;
n = 100L;
prob = rep(n / N, N);
xs = matrix(runif(N * 2), ncol = 2);

# Construct circular dbd design
design = dbd_circular(
  n,
  xs,
  annealing_temp = 0.1,
  annealing_cooling = 0.999,
  max_iter = 1e5L
);
s = draw(design); # Draw sample from design
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

# Construct dbd design using tactical configuration
design = dbd_tc(
  n,
  xs,
  annealing_temp = 0.1,
  annealing_cooling = 0.999,
  max_iter = 1e5L
);
s = draw(design); # Draw sample from design
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

}
\references{
Grafström, A., & Prentius, W. (2026).
Distributionally balanced sampling designs.
arXiv preprint arXiv:2603.11916.

Grafström, A., & Prentius, W. (2026).
Distributionally balanced sampling designs via minimum tactical configurations.
arXiv preprint arXiv:2603.24439.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doubly_balanced_sampling.R
\name{Doubly balanced sampling}
\alias{Doubly balanced sampling}
\alias{local_cube}
\alias{local_cube_stratified}
\title{Doubly balanced sampling}
\usage{
local_cube(probabilities, spread_mat, balance_mat, ...)

local_cube_stratified(probabilities, spread_mat, balance_mat, strata, ...)
}
\arguments{
\item{probabilities}{A vector of inclusion probabilities.}

\item{spread_mat}{A matrix of spreading covariates.}

\item{balance_mat}{A matrix of balancing covariates.}

\item{...}{
  Arguments passed on to \code{\link[=.sampling_defaults]{.sampling_defaults}}
  \describe{
    \item{\code{eps}}{A small value used when comparing floats.}
    \item{\code{bucket_size}}{The maximum size of the k-d-tree nodes. A higher value gives a slower
k-d-tree, but is faster to create and takes up less memory.}
  }}

\item{strata}{An integer vector with stratum numbers for each unit.}
}
\value{
A vector of sample indices.
}
\description{
Selects doubly balanced samples with prescribed inclusion probabilities from finite populations.
}
\details{
For the \code{local_cube} method, a fixed sized sample is obtained if the first column of
\code{balance_mat} is the inclusion probabilities. For \code{local_cube_stratified}, the inclusion
probabilities are inserted automatically.
}
\section{Functions}{
\itemize{
\item \code{local_cube()}: The local cube method

\item \code{local_cube_stratified()}: The stratified local cube method

}}
\examples{
set.seed(12345);
N = 1000;
n = 100;
prob = rep(n/N, N);
xb = matrix(c(prob, runif(N * 2)), ncol = 3);
xs = matrix(runif(N * 2), ncol = 2);
strata = c(rep(1L, 100), rep(2L, 200), rep(3L, 300), rep(4L, 400));

s = local_cube(prob, xs, xb);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = local_cube_stratified(prob, xs, xb[, -1], strata);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

\donttest{
# Respects inclusion probabilities
set.seed(12345);
prob = c(0.2, 0.25, 0.35, 0.4, 0.5, 0.5, 0.55, 0.65, 0.7, 0.9);
N = length(prob);
xb = matrix(c(prob, runif(N * 2)), ncol = 3);
xs = matrix(runif(N * 2), ncol = 2);

ep = rep(0L, N);
r = 10000L;

for (i in seq_len(r)) {
  s = local_cube(prob, xs, xb);
  ep[s] = ep[s] + 1L;
}

print(ep / r - prob);
}

}
\references{
Deville, J. C. and Tillé, Y. (2004).
Efficient balanced sampling: the cube method.
Biometrika, 91(4), 893-912.

Chauvet, G. and Tillé, Y. (2006).
A fast algorithm for balanced sampling.
Computational Statistics, 21(1), 53-62.

Chauvet, G. (2009).
Stratified balanced sampling.
Survey Methodology, 35, 115-119.

Grafström, A. and Tillé, Y. (2013).
Doubly balanced spatial sampling with spreading and restitution of auxiliary totals.
Environmetrics, 24(2), 120-131
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.sampling_defaults}
\alias{.sampling_defaults}
\title{Sampling defaults}
\usage{
.sampling_defaults(eps = 1e-10, max_iter = 1000L, bucket_size = 50L)
}
\arguments{
\item{eps}{A small value used when comparing floats.}

\item{max_iter}{The maximum number of iterations used in iterative algorithms.}

\item{bucket_size}{The maximum size of the k-d-tree nodes. A higher value gives a slower
k-d-tree, but is faster to create and takes up less memory.}
}
\value{
A validated list of arguments used internally in sampling functions.
}
\description{
Sampling defaults
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pips_from_vector.R
\name{pips_from_vector}
\alias{pips_from_vector}
\title{Inclusion probabilities proportional-to-size}
\usage{
pips_from_vector(values, sample_size)
}
\arguments{
\item{values}{A vector of positive numbers}

\item{sample_size}{The wanted sample size}
}
\value{
A vector of inclusion probabilities proportional-to-size.
}
\description{
Computes the first-order inclusion probabilities from a vector of positive numbers,
for an inclusion probabilities proportional-to-size design.
}
\examples{
set.seed(12345);
N = 1000;
n = 100;
x = matrix(runif(N * 2), ncol = 2);
prob = pips_from_vector(x[, 1], n);
s = lpm_2(prob, x);
plot(x[, 1], x[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sample_to_indicator}
\alias{sample_to_indicator}
\title{Transform a sample vector into an inclusion indicator vector}
\usage{
sample_to_indicator(sample, population_size)
}
\arguments{
\item{sample}{A vector of sample indices.}

\item{population_size}{The total size of the population.}
}
\value{
An inclusion indicator vector, i.e. a \code{population_size}-sized vector of 0/1.
}
\description{
Transform a sample vector into an inclusion indicator vector
}
\examples{
s = c(1, 2, 10);
si = sample_to_indicator(s, 10);

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distributional_designs.R
\name{.dbd_defaults}
\alias{.dbd_defaults}
\title{Distributionally balanced designs defaults}
\usage{
.dbd_defaults(
  annealing_temp = 0.1,
  annealing_cooling = 0.999,
  spatial_init = FALSE,
  ...
)
}
\arguments{
\item{annealing_temp}{The initial temperature to use in simulated annealing}

\item{annealing_cooling}{The annealing_cooling rate tu use in simulated annealing}

\item{spatial_init}{If \code{TRUE} a spatial initialization strategy is used}

\item{...}{
  Arguments passed on to \code{\link[=.sampling_defaults]{.sampling_defaults}}
  \describe{
    \item{\code{eps}}{A small value used when comparing floats.}
    \item{\code{max_iter}}{The maximum number of iterations used in iterative algorithms.}
  }}
}
\value{
A validated list of arguments used internally in dbd functions.
}
\description{
Distributionally balanced designs defaults
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial_balance.R
\name{Spatial balance measure}
\alias{Spatial balance measure}
\alias{spatial_balance_local}
\alias{spatial_balance_local_equal}
\alias{spatial_balance_voronoi}
\alias{spatial_balance_voronoi_equal}
\alias{spatial_balance_energy}
\alias{spatial_balance_energy_equal}
\alias{spatial_balance_all}
\alias{spatial_balance_all_equal}
\alias{balance_deviation}
\title{Spatial balance measure}
\usage{
spatial_balance_local(
  sample,
  probabilities,
  spread_mat,
  balance_probabilities = TRUE
)

spatial_balance_local_equal(
  sample,
  spread_mat,
  sample_size = length(sample),
  balance_probabilities = TRUE
)

spatial_balance_voronoi(sample, probabilities, spread_mat)

spatial_balance_voronoi_equal(sample, spread_mat, sample_size = length(sample))

spatial_balance_energy(sample, probabilities, spread_mat)

spatial_balance_energy_equal(sample, spread_mat, sample_size = length(sample))

spatial_balance_all(sample, probabilities, spread_mat)

spatial_balance_all_equal(sample, spread_mat, sample_size = length(sample))

balance_deviation(sample, probabilities, spread_mat)
}
\arguments{
\item{sample}{A vector of sample indices.}

\item{probabilities}{A vector of inclusion probabilities.}

\item{spread_mat}{A matrix of spreading covariates.}

\item{balance_probabilities}{If \code{true} (default), includes the vector of inclusion probabilities
as a balancing variable.}

\item{sample_size}{The sample size}
}
\value{
the measure, or in case of \code{balance_deviation}, the vector of deviations.
}
\description{
Calculates the spatial balance of a sample.
}
\section{Functions}{
\itemize{
\item \code{spatial_balance_local()}: Local spatial balance

\item \code{spatial_balance_local_equal()}: Local spatial balance

\item \code{spatial_balance_voronoi()}: Voronoi spatial balance

\item \code{spatial_balance_voronoi_equal()}: Voronoi spatial balance

\item \code{spatial_balance_energy()}: Energy spatial balance

\item \code{spatial_balance_energy_equal()}: Energy spatial balance

\item \code{spatial_balance_all()}: Energy spatial balance

\item \code{spatial_balance_all_equal()}: Energy spatial balance

\item \code{balance_deviation()}: Balance deviation

}}
\examples{
set.seed(12345);
N = 500;
n = 70;
prob = rep(n / N, N);
xs = matrix(runif(N * 2), ncol = 2);

s = lpm_2(prob, xs);
spatial_balance_voronoi(s, prob, xs);
spatial_balance_local(s, prob, xs, TRUE);
spatial_balance_local(s, prob, xs, FALSE);
spatial_balance_energy(s, prob, xs);
balance_deviation(s, prob, xs);

\donttest{
# Compare SRS
r = 1000L;
sb_v = matrix(0.0, r, 2L);
sb_l = matrix(0.0, r, 4L);
sb_e = matrix(0.0, r, 2L);
bal = matrix(0.0, r, 2L * ncol(xs));

for (i in seq_len(r)) {
  s1 = lpm_2(prob, xs);
  s2 = sample(N, n);
  sb_v[i, ] = c(
    spatial_balance_voronoi(s1, prob, xs),
    spatial_balance_voronoi(s2, prob, xs)
  );
  sb_l[i, ] = c(
    spatial_balance_local(s1, prob, xs, TRUE),
    spatial_balance_local(s2, prob, xs, TRUE),
    spatial_balance_local(s1, prob, xs, FALSE),
    spatial_balance_local(s2, prob, xs, FALSE)
  );
  sb_e[i, ] = c(
    spatial_balance_energy(s1, prob, xs),
    spatial_balance_energy(s2, prob, xs)
  );
  bal[i, ] = c(
    balance_deviation(s1, prob, xs),
    balance_deviation(s2, prob, xs)
  );
}

# Spatial balance measure (voronoi), LPM vs SRS
print(colMeans(sb_v));
# Spatial balance measure (local), LPM vs SRS
print(colMeans(sb_l));
# Spatial balance measure (energy), LPM vs SRS
print(colMeans(sb_e));
# Abs. balance deviation, LPM vs SRS
print(colMeans(abs(bal)));
}

}
\references{
Stevens Jr, D. L., & Olsen, A. R. (2004).
Spatially balanced sampling of natural resources.
Journal of the American statistical Association, 99(465), 262-278.

Grafström, A., Lundström, N.L.P. & Schelin, L. (2012).
Spatially balanced sampling through the Pivotal method.
Biometrics 68(2), 514-520.

Prentius, W., & Grafström, A. (2024).
How to find the best sampling design: A new measure of spatial balance.
Environmetrics, 35(7), e2878.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unequal_probability_sampling.R
\name{Unequal probability sampling}
\alias{Unequal probability sampling}
\alias{rpm}
\alias{spm}
\alias{cps}
\alias{poisson}
\alias{conditional_poisson}
\alias{systematic}
\alias{systematic_random_order}
\alias{brewer}
\alias{pareto}
\alias{sampford}
\title{Unequal probability sampling}
\usage{
rpm(probabilities, ...)

spm(probabilities, ...)

cps(probabilities, ...)

poisson(probabilities, ...)

conditional_poisson(probabilities, sample_size, ...)

systematic(probabilities, ...)

systematic_random_order(probabilities, ...)

brewer(probabilities, ...)

pareto(probabilities, ...)

sampford(probabilities, ...)
}
\arguments{
\item{probabilities}{A vector of inclusion probabilities.}

\item{...}{
  Arguments passed on to \code{\link[=.sampling_defaults]{.sampling_defaults}}
  \describe{
    \item{\code{eps}}{A small value used when comparing floats.}
    \item{\code{max_iter}}{The maximum number of iterations used in iterative algorithms.}
  }}

\item{sample_size}{The wanted sample size}
}
\value{
A vector of sample indices.
}
\description{
Selects samples with prescribed inclusion probabilities from finite populations.
}
\details{
\code{sampford} and \code{conditional_poisson} may return an error if a solution is not found within
\code{max_iter}.
}
\section{Functions}{
\itemize{
\item \code{rpm()}: Random pivotal method

\item \code{spm()}: Sequential pivotal method

\item \code{cps()}: Correlated Poisson sampling

\item \code{poisson()}: Poisson sampling

\item \code{conditional_poisson()}: Conditional Poisson sampling

\item \code{systematic()}: Systematic sampling

\item \code{systematic_random_order()}: Systematic sampling with random order

\item \code{brewer()}: Brewer sampling

\item \code{pareto()}: Pareto sampling

\item \code{sampford()}: Sampford sampling

}}
\examples{
set.seed(12345);
N = 1000;
n = 100;
prob = rep(n/N, N);
xs = matrix(runif(N * 2), ncol = 2);

s = rpm(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = spm(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = cps(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = poisson(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = brewer(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = pareto(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = systematic(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

s = systematic_random_order(prob);
plot(xs[, 1], xs[, 2], pch = ifelse(sample_to_indicator(s, N), 19, 1));

# Conditional poisson and sampford are not guaranteed to find a solution
prob2 = rep(0.5, 10L);
s = conditional_poisson(prob2, 5L, max_iter = 10000L);
plot(xs[1:10, 1], xs[1:10, 2], pch = ifelse(sample_to_indicator(s, 10L), 19, 1));

s = sampford(prob2, max_iter = 10000L);
plot(xs[1:10, 1], xs[1:10, 2], pch = ifelse(sample_to_indicator(s, 10L), 19, 1));

\donttest{
# Respects inclusion probabilities
set.seed(12345);
prob = c(0.2, 0.25, 0.35, 0.4, 0.5, 0.5, 0.55, 0.65, 0.7, 0.9);
N = length(prob);

ep = rep(0L, N);
r = 10000L;

for (i in seq_len(r)) {
  s = poisson(prob);
  ep[s] = ep[s] + 1L;
}

print(ep / r - prob);
}

}
\references{
Bondesson, L., & Thorburn, D. (2008).
A list sequential sampling method suitable for real‐time sampling.
Scandinavian Journal of Statistics, 35(3), 466-483.

Brewer, K. E. (1975).
A Simple Procedure for Sampling pi-ps wor.
Australian Journal of Statistics, 17(3), 166-172.

Chauvet, G. (2012).
On a characterization of ordered pivotal sampling.
Bernoulli, 18(4), 1320-1340.

Deville, J.-C., &  Tillé, Y. (1998).
Unequal probability sampling without replacement through a splitting method.
Biometrika 85, 89-101.

Grafström, A. (2009).
Non-rejective implementations of the Sampford sampling design.
Journal of Statistical Planning and Inference, 139(6), 2111-2114.

Rosén, B. (1997).
On sampling with probability proportional to size.
Journal of statistical planning and inference, 62(2), 159-191.

Sampford, M. R. (1967).
On sampling without replacement with unequal probabilities of selection.
Biometrika, 54(3-4), 499-513.
}
